"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const postgres_utils_1 = require("./postgres_utils");
const velo_external_db_commons_1 = require("@wix-velo/velo-external-db-commons");
const sql_exception_translator_1 = require("./sql_exception_translator");
class DataProvider {
    constructor(pool, filterParser) {
        this.filterParser = filterParser;
        this.pool = pool;
    }
    find(collectionName, filter, sort, skip, limit, projection) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { filterExpr, parameters, offset } = this.filterParser.transform(filter);
            const { sortExpr } = this.filterParser.orderBy(sort);
            const projectionExpr = this.filterParser.selectFieldsFor(projection);
            const sql = `SELECT ${projectionExpr} FROM ${(0, postgres_utils_1.escapeIdentifier)(collectionName)} ${filterExpr} ${sortExpr} OFFSET $${offset} LIMIT $${offset + 1}`;
            const resultSet = yield this.pool.query(sql, [...parameters, skip, limit])
                .catch(err => (0, sql_exception_translator_1.translateErrorCodes)(err, collectionName));
            return resultSet.rows;
        });
    }
    count(collectionName, filter) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { filterExpr, parameters } = this.filterParser.transform(filter);
            const resultSet = yield this.pool.query(`SELECT COUNT(*) AS num FROM ${(0, postgres_utils_1.escapeIdentifier)(collectionName)} ${filterExpr}`, parameters)
                .catch(err => (0, sql_exception_translator_1.translateErrorCodes)(err, collectionName));
            return parseInt(resultSet.rows[0]['num'], 10);
        });
    }
    insert(collectionName, items, fields, upsert) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const itemsAsParams = items.map((item) => (0, velo_external_db_commons_1.asParamArrays)((0, velo_external_db_commons_1.patchDateTime)(item)));
            const escapedFieldsNames = fields.map((f) => (0, postgres_utils_1.escapeIdentifier)(f.field)).join(', ');
            const upsertAddon = upsert ? ` ON CONFLICT (_id) DO UPDATE SET ${fields.map(f => `${(0, postgres_utils_1.escapeIdentifier)(f.field)} = EXCLUDED.${(0, postgres_utils_1.escapeIdentifier)(f.field)}`).join(', ')}` : '';
            const query = `INSERT INTO ${(0, postgres_utils_1.escapeIdentifier)(collectionName)} (${escapedFieldsNames}) VALUES ${(0, postgres_utils_1.prepareStatementVariablesForBulkInsert)(items.length, fields.length)}${upsertAddon}`;
            yield this.pool.query(query, itemsAsParams.flat()).catch(err => (0, sql_exception_translator_1.translateErrorCodes)(err, collectionName));
            return items.length;
        });
    }
    update(collectionName, items) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const updateFields = (0, velo_external_db_commons_1.updateFieldsFor)(items[0]);
            const updatables = items.map((i) => [...updateFields, '_id'].reduce((obj, key) => (Object.assign(Object.assign({}, obj), { [key]: i[key] })), {}))
                .map((u) => (0, velo_external_db_commons_1.asParamArrays)((0, velo_external_db_commons_1.patchDateTime)(u)));
            const res = yield Promise.all(updatables.map((updatable) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const rs = yield this.pool.query(`UPDATE ${(0, postgres_utils_1.escapeIdentifier)(collectionName)} SET ${updateFields.map((f, i) => `${(0, postgres_utils_1.escapeIdentifier)(f)} = $${i + 1}`).join(', ')} WHERE _id = $${updateFields.length + 1}`, updatable)
                    .catch(err => (0, sql_exception_translator_1.translateErrorCodes)(err, collectionName));
                return rs.rowCount;
            })));
            // @ts-ignore
            return res.reduce((sum, i) => i + sum, 0);
        });
    }
    delete(collectionName, itemIds) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const rs = yield this.pool.query(`DELETE FROM ${(0, postgres_utils_1.escapeIdentifier)(collectionName)} WHERE _id IN (${(0, postgres_utils_1.prepareStatementVariables)(itemIds.length)})`, itemIds)
                .catch(err => (0, sql_exception_translator_1.translateErrorCodes)(err, collectionName));
            // @ts-ignore
            return rs.rowCount;
        });
    }
    truncate(collectionName) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.pool.query(`TRUNCATE ${(0, postgres_utils_1.escapeIdentifier)(collectionName)}`).catch(err => (0, sql_exception_translator_1.translateErrorCodes)(err, collectionName));
        });
    }
    aggregate(collectionName, filter, aggregation, sort, skip, limit) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { filterExpr: whereFilterExpr, parameters: whereParameters, offset } = this.filterParser.transform(filter);
            const { fieldsStatement, groupByColumns, havingFilter: filterExpr, parameters: havingParameters, offset: offsetAfterAggregation } = this.filterParser.parseAggregation(aggregation, offset);
            const { sortExpr } = this.filterParser.orderBy(sort);
            const sql = `SELECT ${fieldsStatement} FROM ${(0, postgres_utils_1.escapeIdentifier)(collectionName)} ${whereFilterExpr} GROUP BY ${groupByColumns.map(postgres_utils_1.escapeIdentifier).join(', ')} ${filterExpr} ${sortExpr} OFFSET $${offsetAfterAggregation} LIMIT $${offsetAfterAggregation + 1}`;
            const rs = yield this.pool.query(sql, [...whereParameters, ...havingParameters, skip, limit])
                .catch(err => (0, sql_exception_translator_1.translateErrorCodes)(err, collectionName));
            return rs.rows;
        });
    }
}
exports.default = DataProvider;
//# sourceMappingURL=postgres_data_provider.js.map