"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const velo_external_db_commons_1 = require("@wix-velo/velo-external-db-commons");
const sql_exception_translator_1 = require("./sql_exception_translator");
const sql_schema_translator_1 = require("./sql_schema_translator");
const postgres_utils_1 = require("./postgres_utils");
const velo_external_db_types_1 = require("@wix-velo/velo-external-db-types");
const postgres_capabilities_1 = require("./postgres_capabilities");
const { CollectionDoesNotExists } = velo_external_db_commons_1.errors;
class SchemaProvider {
    constructor(pool) {
        this.pool = pool;
        this.sqlSchemaTranslator = new sql_schema_translator_1.default();
    }
    list() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const data = yield this.pool.query('SELECT table_name, column_name AS field, data_type, udt_name AS type FROM information_schema.columns WHERE table_schema = $1 ORDER BY table_name', ['public']);
            const tables = (0, velo_external_db_commons_1.parseTableData)(data.rows);
            return Object.entries(tables)
                .map(([collectionName, rs]) => ({
                id: collectionName,
                fields: rs.map(this.appendAdditionalRowDetails.bind(this)),
                capabilities: this.collectionCapabilities(rs.map(r => r.field))
            }));
        });
    }
    listHeaders() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const data = yield this.pool.query('SELECT table_name FROM information_schema.tables WHERE table_schema = $1 ORDER BY table_name', ['public']);
            return data.rows.map(rs => rs.table_name);
        });
    }
    supportedOperations() {
        return velo_external_db_commons_1.AllSchemaOperations;
    }
    create(collectionName, columns) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const dbColumnsSql = columns.map(c => this.sqlSchemaTranslator.columnToDbColumnSql(c)).join(', ');
            const primaryKeySql = columns.filter(f => f.isPrimary).map(f => (0, postgres_utils_1.escapeIdentifier)(f.name)).join(', ');
            yield this.pool.query(`CREATE TABLE IF NOT EXISTS ${(0, postgres_utils_1.escapeIdentifier)(collectionName)} (${dbColumnsSql}, PRIMARY KEY (${primaryKeySql}))`)
                .catch(err => (0, sql_exception_translator_1.translateErrorCodes)(err, collectionName));
        });
    }
    drop(collectionName) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.pool.query(`DROP TABLE IF EXISTS ${(0, postgres_utils_1.escapeIdentifier)(collectionName)}`)
                .catch(err => (0, sql_exception_translator_1.translateErrorCodes)(err, collectionName));
        });
    }
    addColumn(collectionName, column) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield (0, velo_external_db_commons_1.validateSystemFields)(column.name);
            yield this.pool.query(`ALTER TABLE ${(0, postgres_utils_1.escapeIdentifier)(collectionName)} ADD ${(0, postgres_utils_1.escapeIdentifier)(column.name)} ${this.sqlSchemaTranslator.dbTypeFor(column)}`)
                .catch(err => (0, sql_exception_translator_1.translateErrorCodes)(err, collectionName));
        });
    }
    changeColumnType(collectionName, column) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield (0, velo_external_db_commons_1.validateSystemFields)(column.name);
            const query = `ALTER TABLE ${(0, postgres_utils_1.escapeIdentifier)(collectionName)} ALTER COLUMN ${(0, postgres_utils_1.escapeIdentifier)(column.name)} TYPE ${this.sqlSchemaTranslator.dbTypeFor(column)} USING (${(0, postgres_utils_1.escapeIdentifier)(column.name)}::${this.sqlSchemaTranslator.dbTypeFor(column)})`;
            yield this.pool.query(query)
                .catch(err => (0, sql_exception_translator_1.translateErrorCodes)(err, collectionName, column.name));
        });
    }
    removeColumn(collectionName, columnName) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield (0, velo_external_db_commons_1.validateSystemFields)(columnName);
            yield this.pool.query(`ALTER TABLE ${(0, postgres_utils_1.escapeIdentifier)(collectionName)} DROP COLUMN ${(0, postgres_utils_1.escapeIdentifier)(columnName)}`)
                .catch(err => (0, sql_exception_translator_1.translateErrorCodes)(err, collectionName));
        });
    }
    describeCollection(collectionName) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const res = yield this.pool.query('SELECT table_name, column_name AS field, data_type, udt_name AS type, character_maximum_length FROM information_schema.columns WHERE table_schema = $1 AND table_name = $2 ORDER BY table_name', ['public', collectionName])
                .catch(err => (0, sql_exception_translator_1.translateErrorCodes)(err, collectionName));
            if (res.rows.length === 0) {
                throw new CollectionDoesNotExists('Collection does not exists', collectionName);
            }
            const fields = res.rows.map(r => ({ field: r.field, type: r.type })).map(r => this.appendAdditionalRowDetails(r));
            return {
                id: collectionName,
                fields,
                capabilities: this.collectionCapabilities(res.rows.map(r => r.field))
            };
        });
    }
    collectionCapabilities(fieldNames) {
        return {
            dataOperations: fieldNames.includes('_id') ? postgres_capabilities_1.ReadWriteOperations : postgres_capabilities_1.ReadOnlyOperations,
            fieldTypes: postgres_capabilities_1.FieldTypes,
            collectionOperations: postgres_capabilities_1.CollectionOperations,
            referenceCapabilities: { supportedNamespaces: [] },
            indexing: [],
            encryption: velo_external_db_types_1.Encryption.notSupported,
            pagingMode: velo_external_db_types_1.PagingMode.offset
        };
    }
    appendAdditionalRowDetails(row) {
        var _a;
        const type = this.sqlSchemaTranslator.translateType(row.type);
        return Object.assign(Object.assign({}, row), { type: this.sqlSchemaTranslator.translateType(row.type), capabilities: (_a = postgres_capabilities_1.ColumnsCapabilities[type]) !== null && _a !== void 0 ? _a : velo_external_db_commons_1.EmptyCapabilities });
    }
    translateDbTypes(row) {
        return {
            field: row.field,
            type: this.sqlSchemaTranslator.translateType(row.type)
        };
    }
}
exports.default = SchemaProvider;
//# sourceMappingURL=postgres_schema_provider.js.map